#include "libelle_eve_objects.h"
#include "libelle_eve_resources.h"
#include "built_in.h"
#include "FT800_Types.h"


// TFT module connections
sbit FT800_RST at LATC1_bit;
sbit FT800_CS at LATC2_bit;
sbit FT800_RST_Direction at TRISC1_bit;
sbit FT800_CS_Direction at TRISC2_bit;
// End TFT module connections

// Object type constants
// Usage: VTFT stack internally
const VTFT_OT_LABEL     = 4;
const VTFT_OT_CIRCLE    = 8;
const VTFT_OT_EVESLIDER = 32;
const VTFT_OT_EVETEXT   = 40;
// ~Object type constants

// Event type constants
// Usage: OnEvent
const VTFT_EVT_UP    = 0;
const VTFT_EVT_DOWN  = 1;
const VTFT_EVT_CLICK = 2;
const VTFT_EVT_PRESS = 3;
// ~Event type constants

// Sound action constants
// Usage: sound event action property and ProcessEvent
const VTFT_SNDACT_NONE  = 0;
const VTFT_SNDACT_PLAY  = 1;
const VTFT_SNDACT_STOP  = 2;
// ~Sound action constants

// Resource loading constants.
// Usage: DrawScreenO and LoadCurrentScreenResToGRAM
const VTFT_LOAD_RES_NONE    = 0x00; // do not load g-ram resources
const VTFT_LOAD_RES_STATIC  = 0x01; // load g-ram resources for static objects
const VTFT_LOAD_RES_DYNAMIC = 0x02; // load g-ram resources for dynamic objects
const VTFT_LOAD_RES_ALL     = 0x03; // load g-ram resources for all objects
// ~Resource loading constants

// Display effect constants
// Usage: DrawScreenO
const VTFT_DISPLAY_EFF_NONE         = 0x00; // no effect when switching between screens
const VTFT_DISPLAY_EFF_LIGHTS_FADE  = 0x04; // backlight: fade out before, fade in after drawing new screen
const VTFT_DISPLAY_EFF_LIGHTS_OFF   = 0x08; // backlight: turn off before, turn on after drawing new screen
// ~Display effect constants

// Stack flags
// Usage: internally used by VTFT stack
const VTFT_INT_REPAINT_ON_DOWN     = 1 << 0;
const VTFT_INT_REPAINT_ON_UP       = 1 << 1;
const VTFT_INT_BRING_TO_FRONT      = 1 << 2;
const VTFT_INT_INTRINSIC_CLICK_EFF = 1 << 3;
// ~Stack flags

// Table of object draw handlers
// Use object type constants to access coresponding object draw handler
const TPointer DrawHandlerTable[44] = {
  0,               // Button draw handler not used
  0,               // CButton draw handler not used
  0,               // ButtonRound draw handler not used
  0,               // CButtonRound draw handler not used
  &DrawLabel,      // Label draw handler
  0,               // CLabel draw handler not used
  0,               // Image draw handler not used
  0,               // CImage draw handler not used
  &DrawCircle,     // Circle draw handler
  0,               // CCircle draw handler not used
  0,               // CircleButton draw handler not used
  0,               // CCircleButton draw handler not used
  0,               // Box draw handler not used
  0,               // CBox draw handler not used
  0,               // BoxRound draw handler not used
  0,               // CBoxRound draw handler not used
  0,               // Line draw handler not used
  0,               // CLine draw handler not used
  0,               // Polygon draw handler not used
  0,               // CPolygon draw handler not used
  0,               // CheckBox draw handler not used
  0,               // RadioButton draw handler not used
  0,               // ProgressBar draw handler not used
  0,               // Audio draw handler not used
  0,               // EveClock draw handler not used
  0,               // EveGauge draw handler not used
  0,               // EveDial draw handler not used
  0,               // EveKeys draw handler not used
  0,               // CEveKeys draw handler not used
  0,               // EveProgressBar draw handler not used
  0,               // EveScrollBar draw handler not used
  0,               // EveToggle draw handler not used
  &DrawEveSlider,  // EveSlider draw handler
  0,               // EveSpinner draw handler not used
  0,               // EveScreenSaver draw handler not used
  0,               // EveScatch draw handler not used
  0,               // EveButton draw handler not used
  0,               // CEveButton draw handler not used
  0,               // EveGradient draw handler not used
  0,               // CEveGradient draw handler not used
  &DrawEveText,    // EveText draw handler
  0,               // CEveText draw handler not used
  0,               // EveNumber draw handler not used
  0                // CEveNumber draw handler not used
};
// ~Table of draw handler pointers

// Default configuration parameters
const TFT800Display VTFT_FT800_CONFIG_DISPLAY =
{
  48000000,        // Frequency          = main clock frequency
  0,               // OutRenderMode      = 0 normal, 1 write, 2 read
  0,               // RenderReadScanLine = scanline for read render mode
  0,               // RenderWriteTrigger = trigger for write render mode (read only)
  525,             // hCycle             = number if horizontal cycles for display
  43,              // hOffset            = horizontal offset from starting signal
  480,             // hSize              = width resolution
  0,               // hSync0             = hsync falls
  41,              // hSync1             = hsync rise
  286,             // vCycle             = number of vertical cycles for display
  12,              // vOffset            = vertical offset from start signal
  272,             // vSize              = height resolution
  0,               // vSync0             = vsync falls
  10,              // vSync1             = vsync rise
  0,               // Rotate180          = rotate display 180 degrees: 1 - rotate enable, 0 - rotate disable
  0x01B6,          // OutBits            = output bits resolution
  0,               // OutDither          = output number of bits
  0x0000,          // OutSwizzle         = output swizzle
  0,               // OutCSpread         = output clock spread enable
  1,               // PClockPolarity     = clock polarity: 0 - rising edge, 1 - falling edge
  5,               // PClock             = clock prescaler of FT800: - 0 means disable and >0 means 48MHz/pclock
};

const TFT800Touch VTFT_FT800_CONFIG_TOUCH =
{
  3,               // TouchMode        = touch screen mode (2 bits): 0 - off, 1 - oneshot, 2 - frame, 3 - continuous
  1,               // TouchADCMode     = touch screen adc mode (1 bit): 0 - singleended, 1 - differential
  5000,            // TouchCharge      = Touchscreen charge time, units of 6 clocks
  3,               // TouchSettle      = touch screen settle time - 4 bits
  7,               // TouchOversample  = touch screen oversample - 4 bits
  1500,            // TouchRZThreshold = Touchscreen resistance threshold
};

const TFT800TouchTransform VTFT_FT800_CONFIG_TOUCHTRANSFORM =
{
  0x00008128,      // TransformA = touch matrix transformation coefficient A
  0xFFFFFe4d,      // TransformB = touch matrix transformation coefficient B
  0xFFED5516,      // TransformC = touch matrix transformation coefficient C
  0xFFFFFE94,      // TransformD = touch matrix transformation coefficient D
  0x00004C78,      // TransformE = touch matrix transformation coefficient E
  0xFFF095AB,      // TransformF = touch matrix transformation coefficient F
};

const TFT800GPIO VTFT_FT800_CONFIG_GPIO =
{
  0xFC,            // GPIODIR = GPIO direction: 1 - output, 0 - input (8bit wide)
  0xFF,            // GPIO    = GPIO data latch
};

const TFT800PWM VTFT_FT800_CONFIG_PWM =
{
  250,             // Freq = PWM frequency - 14 bits
  128,             // Duty = PWM duty cycle, 0 to 128 is the range
};

const TFT800Interrupt VTFT_FT800_CONFIG_INTERRUPT =
{
  0,               // Enable = global interrupt enable: 1 - enabled, 0 - disabled
  255,             // Mask   = interrupt mask value (individual interrupt enable): 1 - masked/disabled, 0 - enabled
  0,               // Flags  = interrupt flags (read only)
};

const TFT800Sound VTFT_FT800_CONFIG_SOUND =
{
  0,               // Effect
  0,               // Pitch
  0,               // Volume
  0,               // Play
};

const TFT800Audio VTFT_FT800_CONFIG_AUDIO =
{
  0,               // StartAddress
  0,               // Length
  8000,            // Frequency
  0,               // Format
  0,               // Loop = audio playback mode
  0,               // Volume
  0,               // Play
};

// Global variables

TTouchStat TouchS = {0};


/////////////////////////
TScreen *CurrentScreen = 0;


TScreen Screen2;

TEveSlider EveSlider_Basisachse;
TEvent     EveSlider_Basisachse_OnPress;
TEveSlider EveSlider_Achse2;
TEvent     EveSlider_Achse2_OnPress;
TCircle    Circle2;
TCircle    Circle4;
TCircle    Circle_Blase;
TEveText   EveText2;
char       EveText2_Caption[11] = "Basisachse";
TEveText   EveText3;
char       EveText3_Caption[8] = "Achse 2";
TLabel     Label10;
char       Label10_Caption[38] = "center = from left 320 / from top 120";
TLabel     Label3;
char       Label3_Caption[9] = "Text";
TLabel     Label4;
char       Label4_Caption[9] = "Text";
TLabel     Label1;
char       Label1_Caption[9] = "Text";
TLabel     Label5;
char       Label5_Caption[9] = "Text";
TLabel     Label6;
char       Label6_Caption[9] = "Text";
TLabel     Label2;
char       Label2_Caption[9] = "Text";
TLabel     Label7;
char       Label7_Caption[2] = "C";
TLabel     Label8;
char       Label8_Caption[2] = "A";
TLabel     Label9;
char       Label9_Caption[2] = "B";
TLabel     Label11;
char       Label11_Caption[2] = "F";
TLabel     Label12;
char       Label12_Caption[2] = "D";
TLabel     Label13;
char       Label13_Caption[2] = "E";
TLabel     Label14;
char       Label14_Caption[25] = "Scanned touch variables:";

TLabel *const code Screen2_Labels[14] = {
  &Label10,             
  &Label3,              
  &Label4,              
  &Label1,              
  &Label5,              
  &Label6,              
  &Label2,              
  &Label7,              
  &Label8,              
  &Label9,              
  &Label11,             
  &Label12,             
  &Label13,             
  &Label14              
};

TCircle *const code Screen2_Circles[3] = {
  &Circle2,             
  &Circle4,             
  &Circle_Blase         
};

TEveSlider *const code Screen2_EveSliders[2] = {
  &EveSlider_Basisachse,
  &EveSlider_Achse2     
};

TEveText *const code Screen2_EveTexts[2] = {
  &EveText2,            
  &EveText3             
};


static char IsInsideObject(TObjInfo *AObjInfo, unsigned int X, unsigned int Y) {
  TRect *ptrPressRect = 0;
  TRect *ptrPressCircle = 0;

  if ((AObjInfo->HitX == X) && (AObjInfo->HitY == Y))
    return 1;

  switch (AObjInfo->Type) {
    // Label
    case VTFT_OT_LABEL: {
      ptrPressRect = (TRect *)&(((TLabel *)AObjInfo->Obj)->Left);
      break;
    }
    // Circle
    case VTFT_OT_CIRCLE: {
      ptrPressCircle = (TRect *)&(((TCircle *)AObjInfo->Obj)->Left);
      break;
    }
    // EveSlider
    case VTFT_OT_EVESLIDER: {
      ptrPressRect = (TRect *)&(((TEveSlider *)AObjInfo->Obj)->Left);
      break;
    }
    // EveText
    case VTFT_OT_EVETEXT: {
      ptrPressRect = (TRect *)&(((TEveText *)AObjInfo->Obj)->Left);
      break;
    }
  }

  if (ptrPressRect) {
    if ((ptrPressRect->Left <= X) && (ptrPressRect->Left+ptrPressRect->Width-1 >= X) &&
        (ptrPressRect->Top  <= Y) && (ptrPressRect->Top+ptrPressRect->Height-1 >= Y))
      return 1;
    }
  else if (ptrPressCircle) {
    if ((ptrPressCircle->Left <= X) && (ptrPressCircle->Left+ptrPressCircle->Width*2-1 >= X) &&
        (ptrPressCircle->Top  <= Y) && (ptrPressCircle->Top+ptrPressCircle->Width*2-1 >= Y))
      return 1;
    }

  return 0;
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible) {
    if (ALabel->FontHandle >= 16)
      FT800_Canvas_FontSystem(ALabel->FontHandle, ALabel->Font_Color, ALabel->Opacity);
    else
      FT800_Canvas_Font(ALabel->FontHandle, ALabel->FontName, ALabel->Source, ALabel->Font_Color, ALabel->Opacity);

    if (ALabel->Tag)
      FT800_Canvas_Tag(ALabel->Tag);

    FT800_Screen_TextPos(ALabel->Left, ALabel->Top, ALabel->Caption);
  }
}

void DrawCircle(TCircle *ACircle) {
  if (ACircle->Visible) {
    if (ACircle == (TCircle *)TouchS.ActObjInfo.Obj)
      FT800_Canvas_BrushSingleColor(_FT800_BRUSH_STYLE_SOLID, ACircle->Press_Color, ACircle->Opacity);
    else 
      FT800_Canvas_BrushSingleColor(_FT800_BRUSH_STYLE_SOLID, ACircle->Color, ACircle->Opacity);

    FT800_Canvas_Pen(ACircle->Pen_Width, ACircle->Pen_Color, ACircle->Opacity);

    if (ACircle->Tag)
      FT800_Canvas_Tag(ACircle->Tag);

    FT800_Screen_Circle(ACircle->Left+ACircle->Radius, 
                      ACircle->Top+ACircle->Radius, 
                      ACircle->Radius);
  }
}

void DrawEveSlider(TEveSlider *AEveSlider) {
  unsigned long drawOptions;
  
  if (AEveSlider->Visible) {
    if (AEveSlider == (TEveSlider *)TouchS.ActObjInfo.Obj)
      FT800_Canvas_BrushDualColor(_FT800_BRUSH_STYLE_SOLID, AEveSlider->Press_Color, AEveSlider->Thumb_Color, AEveSlider->Opacity);
    else 
      FT800_Canvas_BrushDualColor(_FT800_BRUSH_STYLE_SOLID, AEveSlider->Background_Color, AEveSlider->Thumb_Color, AEveSlider->Opacity);

    FT800_Canvas_Pen(1, AEveSlider->Color, AEveSlider->Opacity);

    FT800_Canvas_CPGraphics_Range(AEveSlider->Range);

    FT800_Canvas_CPGraphics_Value(AEveSlider->Value);

    if (AEveSlider->Tag)
      FT800_Canvas_Tag(AEveSlider->Tag);

    drawOptions = 0;
    if (AEveSlider->Flat)
      drawOptions |= _FT800_CP_DRAW_OPT_FLAT;
    FT800_Canvas_CPGraphics_DrawingOptions(drawOptions);

    FT800_Screen_SliderCP(AEveSlider->Left, AEveSlider->Top, AEveSlider->Width, AEveSlider->Height);
  }
}

void DrawEveText(TEveText *AEveText) {
  unsigned long drawOptions;
  
  if (AEveText->Visible) {
    if (AEveText->FontHandle >= 16)
      FT800_Canvas_FontSystem(AEveText->FontHandle, AEveText->Font_Color, AEveText->Opacity);
    else
      FT800_Canvas_Font(AEveText->FontHandle, AEveText->FontName, AEveText->Source, AEveText->Font_Color, AEveText->Opacity);

    if (AEveText->Tag)
      FT800_Canvas_Tag(AEveText->Tag);

    drawOptions = 0;
    if (AEveText->TextAlign == taCenter)
      drawOptions |= _FT800_CP_DRAW_OPT_CENTER;
    else if (AEveText->TextAlign == taCenterX)
      drawOptions |= _FT800_CP_DRAW_OPT_CENTERX;
    else if (AEveText->TextAlign == taCenterY)
      drawOptions |= _FT800_CP_DRAW_OPT_CENTERY;
    else if (AEveText->TextAlign == taRightX)
      drawOptions |= _FT800_CP_DRAW_OPT_RIGHTX;
    FT800_Canvas_CPGraphics_DrawingOptions(drawOptions);

    FT800_Screen_TextCP(AEveText->Left, AEveText->Top, AEveText->Caption);
  }
}

void SetIdenticalFontSources(const code char *AFontName, unsigned long ASource) {
  char i;
  TLabel   *pLabel;
  TEveText *pEveText;
  void *const code *ptrO;

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    if (pLabel->FontName == AFontName)
      pLabel->Source = ASource;
    ptrO++;
  }

  // EveText
  i    = CurrentScreen->EveTextsCount;
  ptrO = CurrentScreen->EveTexts;
  while (i--) {
    pEveText = (TEveText *)(*ptrO);
    if (pEveText->FontName == AFontName)
      pEveText->Source = ASource;
    ptrO++;
  }
}

static void ClearDynObjSource() {
  char i;
  TLabel   *pLabel;
  TEveText *pEveText;
  void *const code *ptrO;

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    pLabel->Source = 0xFFFFFFFF;
    ptrO++;
  }

  // EveText
  i    = CurrentScreen->EveTextsCount;
  ptrO = CurrentScreen->EveTexts;
  while (i--) {
    pEveText = (TEveText *)(*ptrO);
    pEveText->Source = 0xFFFFFFFF;
    ptrO++;
  }
}

void LoadCurrentScreenResToGRAM(char loadOptions) {
  char i;
  long currSource = -1;
  TLabel   *pLabel;
  TEveText *pEveText;
  void *const code *ptrO;

  // clear dynamic resource addresses first, if necessary
  if (loadOptions & VTFT_LOAD_RES_DYNAMIC)
    ClearDynObjSource();

  // dynamic resources allocation
  if (loadOptions & VTFT_LOAD_RES_DYNAMIC) {

    if (FT800_RES_BeginLoad(CurrentScreen->DynResStart)) {
      return;
    }

    // Label
    i    = CurrentScreen->LabelsCount;
    ptrO = CurrentScreen->Labels;
    while (i--) {
      pLabel = (TLabel *)(*ptrO);
      if (pLabel->FontHandle < 16)
        if (pLabel->Source == 0xFFFFFFFF) {
          currSource = FT800_Res_LoadFont(pLabel->FontName);
          pLabel->Source = currSource;
          SetIdenticalFontSources(pLabel->FontName, currSource);
        }
      ptrO++;
    }

    // EveText
    i    = CurrentScreen->EveTextsCount;
    ptrO = CurrentScreen->EveTexts;
    while (i--) {
      pEveText = (TEveText *)(*ptrO);
      if (pEveText->FontHandle < 16)
        if (pEveText->Source == 0xFFFFFFFF) {
          currSource = FT800_Res_LoadFont(pEveText->FontName);
          pEveText->Source = currSource;
          SetIdenticalFontSources(pEveText->FontName, currSource);
        }
      ptrO++;
    }

    FT800_RES_EndLoad();
  }
}

void DrawObject(TPointer aObj, char aObjType) {
  TDrawHandler drawHandler;

  drawHandler = DrawHandlerTable[aObjType];
  if (drawHandler)
    drawHandler(aObj);
}

void DrawScreenO(TScreen *aScreen, char aOptions) {
  unsigned short cOrder, saveOrder;
  signed   int   actObjOrder;
  unsigned short pwmDuty;
  // counter variables
  char cntLabel;
  char cntCircle;
  char cntEveSlider;
  char cntEveText;
  // pointer variables
  TLabel     *const code *pLabel;
  TCircle    *const code *pCircle;
  TEveSlider *const code *pEveSlider;
  TEveText   *const code *pEveText;

  // process screen switching effects
  if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_FADE) {
    FT800_PWM_Get(0, &pwmDuty);
    FT800_PWM_FadeOut(pwmDuty, 0, pwmDuty/32? pwmDuty/32 : 1, 1);
  }
  else if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_OFF) {
    FT800_PWM_Get(0, &pwmDuty);
    FT800_PWM_Duty(0);
  }

  // clear active object when drawing to new screen
  if (CurrentScreen != aScreen)
    memset(&TouchS.ActObjInfo, 0, sizeof(TObjInfo));

  CurrentScreen = aScreen;

  LoadCurrentScreenResToGRAM(aOptions);

  // init counter variables
  cntLabel     = CurrentScreen->LabelsCount;
  cntCircle    = CurrentScreen->CirclesCount;
  cntEveSlider = CurrentScreen->EveSlidersCount;
  cntEveText   = CurrentScreen->EveTextsCount;
  // init pointer variables
  pLabel     = CurrentScreen->Labels;
  pCircle    = CurrentScreen->Circles;
  pEveSlider = CurrentScreen->EveSliders;
  pEveText   = CurrentScreen->EveTexts;

  FT800_Screen_BeginUpdateCP();
  FT800_Canvas_BrushSingleColor(_FT800_BRUSH_STYLE_SOLID, CurrentScreen->Color, 255);
  FT800_Canvas_Tag(0);
  FT800_Screen_Clear(_FT800_CLEAR_ALL);

  actObjOrder = -1;
  if (TouchS.ActObjInfo.Obj)
    if (TouchS.ActObjInfo.Flags & VTFT_INT_BRING_TO_FRONT)
      actObjOrder = TouchS.ActObjInfo.Order;

  cOrder = 0;
  while (cOrder < CurrentScreen->ObjectsCount) {
    saveOrder = cOrder;
    if (pLabel) {
      while ((*pLabel)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawLabel(*pLabel);
        cOrder++;
        pLabel++;
        cntLabel--;
        if (!cntLabel) {
          pLabel = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pCircle) {
      while ((*pCircle)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawCircle(*pCircle);
        cOrder++;
        pCircle++;
        cntCircle--;
        if (!cntCircle) {
          pCircle = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pEveSlider) {
      while ((*pEveSlider)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawEveSlider(*pEveSlider);
        cOrder++;
        pEveSlider++;
        cntEveSlider--;
        if (!cntEveSlider) {
          pEveSlider = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pEveText) {
      while ((*pEveText)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawEveText(*pEveText);
        cOrder++;
        pEveText++;
        cntEveText--;
        if (!cntEveText) {
          pEveText = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    cOrder++;
  }

  // draw pressed object now
  if (TouchS.ActObjInfo.Obj)
    DrawObject(TouchS.ActObjInfo.Obj, TouchS.ActObjInfo.Type);

  FT800_Screen_EndUpdate();

  FT800_Screen_Show();

  // process screen switching effects
  if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_FADE) {
    FT800_PWM_FadeIn(0, pwmDuty, 1, 3);
  }
  else if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_OFF) {
    FT800_PWM_Duty(pwmDuty);
  }

}

void DrawScreen(TScreen *aScreen) {
  if (aScreen != CurrentScreen)
    DrawScreenO(aScreen, VTFT_LOAD_RES_ALL | VTFT_DISPLAY_EFF_LIGHTS_FADE);
  else
    DrawScreenO(aScreen, VTFT_LOAD_RES_NONE);
}

char GetActiveObjectByXY(int X, int Y, TObjInfo *AObjInfo) {
  char i;
  int  hiOrder;
  TLabel     *pLabel;
  TCircle    *pCircle;
  TEveSlider *pEveSlider;
  TEveText   *pEveText;
  void *const code *ptrO;

  // clear current object info
  memset(AObjInfo, 0, sizeof(TObjInfo));

  // Find object with highest order at specified position.
  // Objects lists are sorted by object order ascending.
  hiOrder = -1;

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels+CurrentScreen->LabelsCount-1;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    if (pLabel->Order < hiOrder)
      break;
    if (pLabel->Active) {
      if ((pLabel->Left <= X) && (pLabel->Left+pLabel->Width-1 >= X) &&
          (pLabel->Top  <= Y) && (pLabel->Top+pLabel->Height-1 >= Y)) {
          AObjInfo->Obj   = (void *)pLabel;
          AObjInfo->Type  = VTFT_OT_LABEL;
          AObjInfo->Order = pLabel->Order;
          AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
          hiOrder         = pLabel->Order;
          break;
      }
    }
    ptrO--;
  }

  // Circle
  i    = CurrentScreen->CirclesCount;
  ptrO = CurrentScreen->Circles+CurrentScreen->CirclesCount-1;
  while (i--) {
    pCircle = (TCircle *)(*ptrO);
    if (pCircle->Order < hiOrder)
      break;
    if (pCircle->Active) {
      // for circle objects common object width is its radius property
      if ((pCircle->Left <= X) && (pCircle->Left+pCircle->Radius*2-1 >= X) &&
          (pCircle->Top  <= Y) && (pCircle->Top+pCircle->Radius*2-1 >= Y)) {
          AObjInfo->Obj   = (void *)pCircle;
          AObjInfo->Type  = VTFT_OT_CIRCLE;
          AObjInfo->Order = pCircle->Order;
          AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if (pCircle->Press_Color != pCircle->Color)
              AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;
          hiOrder         = pCircle->Order;
          break;
      }
    }
    ptrO--;
  }

  // EveSlider
  i    = CurrentScreen->EveSlidersCount;
  ptrO = CurrentScreen->EveSliders+CurrentScreen->EveSlidersCount-1;
  while (i--) {
    pEveSlider = (TEveSlider *)(*ptrO);
    if (pEveSlider->Order < hiOrder)
      break;
    if (pEveSlider->Active) {
      if ((pEveSlider->Left <= X) && (pEveSlider->Left+pEveSlider->Width-1 >= X) &&
          (pEveSlider->Top  <= Y) && (pEveSlider->Top+pEveSlider->Height-1 >= Y)) {
          AObjInfo->Obj   = (void *)pEveSlider;
          AObjInfo->Type  = VTFT_OT_EVESLIDER;
          AObjInfo->Order = pEveSlider->Order;
          AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if (pEveSlider->Press_Color != pEveSlider->Color)
              AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;
          hiOrder         = pEveSlider->Order;
          break;
      }
    }
    ptrO--;
  }

  // EveText
  i    = CurrentScreen->EveTextsCount;
  ptrO = CurrentScreen->EveTexts+CurrentScreen->EveTextsCount-1;
  while (i--) {
    pEveText = (TEveText *)(*ptrO);
    if (pEveText->Order < hiOrder)
      break;
    if (pEveText->Active) {
      if ((pEveText->Left <= X) && (pEveText->Left+pEveText->Width-1 >= X) &&
          (pEveText->Top  <= Y) && (pEveText->Top+pEveText->Height-1 >= Y)) {
          AObjInfo->Obj   = (void *)pEveText;
          AObjInfo->Type  = VTFT_OT_EVETEXT;
          AObjInfo->Order = pEveText->Order;
          AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
          hiOrder         = pEveText->Order;
          break;
      }
    }
    ptrO--;
  }

  if (AObjInfo->Obj) {
    AObjInfo->HitX = X;
    AObjInfo->HitY = Y;
    return 1;
  }
  else {
    return 0;
  }
}

char GetActiveObjectByTag(char ATag, TObjInfo *AObjInfo) {
  char i;
  TLabel     *pLabel;
  TCircle    *pCircle;
  TEveSlider *pEveSlider;
  TEveText   *pEveText;
  void *const code *ptrO;

  // clear current object info
  memset(AObjInfo, 0, sizeof(TObjInfo));

  // Find object with specified tag value.

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels+CurrentScreen->LabelsCount-1;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    if (pLabel->Tag == ATag) {
      if (pLabel->Active) {
        AObjInfo->Obj   = (void *)pLabel;
        AObjInfo->Type  = VTFT_OT_LABEL;
        AObjInfo->Order = pLabel->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
      }
      break;
    }
    ptrO--;
  }

  // Circle
  i    = CurrentScreen->CirclesCount;
  ptrO = CurrentScreen->Circles+CurrentScreen->CirclesCount-1;
  while (i--) {
    pCircle = (TCircle *)(*ptrO);
    if (pCircle->Tag == ATag) {
      if (pCircle->Active) {
        AObjInfo->Obj   = (void *)pCircle;
        AObjInfo->Type  = VTFT_OT_CIRCLE;
        AObjInfo->Order = pCircle->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if (pCircle->Press_Color != pCircle->Color)
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

      }
      break;
    }
    ptrO--;
  }

  // EveSlider
  i    = CurrentScreen->EveSlidersCount;
  ptrO = CurrentScreen->EveSliders+CurrentScreen->EveSlidersCount-1;
  while (i--) {
    pEveSlider = (TEveSlider *)(*ptrO);
    if (pEveSlider->Tag == ATag) {
      if (pEveSlider->Active) {
        AObjInfo->Obj   = (void *)pEveSlider;
        AObjInfo->Type  = VTFT_OT_EVESLIDER;
        AObjInfo->Order = pEveSlider->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if (pEveSlider->Press_Color != pEveSlider->Color)
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

      }
      break;
    }
    ptrO--;
  }

  // EveText
  i    = CurrentScreen->EveTextsCount;
  ptrO = CurrentScreen->EveTexts+CurrentScreen->EveTextsCount-1;
  while (i--) {
    pEveText = (TEveText *)(*ptrO);
    if (pEveText->Tag == ATag) {
      if (pEveText->Active) {
        AObjInfo->Obj   = (void *)pEveText;
        AObjInfo->Type  = VTFT_OT_EVETEXT;
        AObjInfo->Order = pEveText->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
      }
      break;
    }
    ptrO--;
  }

  if (AObjInfo->Obj) {
    AObjInfo->HitTag = ATag;
    return 1;
  }
  else {
    return 0;
  }
}

static void ProcessEvent(TEvent *pEvent) {
  if (pEvent) {
    if (pEvent->Sound.SndAct == VTFT_SNDACT_PLAY) 
      FT800_Sound_SetAndPlay(pEvent->Sound.Effect, pEvent->Sound.Pitch, pEvent->Sound.Volume);
    else if (pEvent->Sound.SndAct == VTFT_SNDACT_STOP) 
      FT800_Sound_Stop();
    if (pEvent->Action) 
      pEvent->Action();
  }
}

static void ProcessCEvent(TCEvent *pEventC) {
  if (pEventC) {
    if (pEventC->Sound.SndAct == VTFT_SNDACT_PLAY) 
      FT800_Sound_SetAndPlay(pEventC->Sound.Effect, pEventC->Sound.Pitch, pEventC->Sound.Volume);
    else if (pEventC->Sound.SndAct == VTFT_SNDACT_STOP) 
      FT800_Sound_Stop();
    if (pEventC->Action) 
      pEventC->Action();
  }
}

static void OnEvent(TObjInfo *AObjInfo, char AEventType){
  TEvent **ppEvent;
  TEvent  *pEvent = 0;

  switch (AObjInfo->Type) {
    // Label
    case VTFT_OT_LABEL: {
      ppEvent = &(((TLabel *)AObjInfo->Obj)->OnUp);
      pEvent  = ppEvent[AEventType];
      break;
    }
    // Circle
    case VTFT_OT_CIRCLE: {
      ppEvent = &(((TCircle *)AObjInfo->Obj)->OnUp);
      pEvent  = ppEvent[AEventType];
      break;
    }
    // EveSlider
    case VTFT_OT_EVESLIDER: {
      ppEvent = &(((TEveSlider *)AObjInfo->Obj)->OnUp);
      pEvent  = ppEvent[AEventType];
      break;
    }
    // EveText
    case VTFT_OT_EVETEXT: {
      ppEvent = &(((TEveText *)AObjInfo->Obj)->OnUp);
      pEvent  = ppEvent[AEventType];
      break;
    }
  } // end switch

  if (pEvent) {
    ProcessEvent(pEvent);
  }
}

static void Process_TP_Press() {
  // Screen Event
  if (CurrentScreen->Active)
    if ((CurrentScreen->SniffObjectEvents) || (!TouchS.ActObjInfo.Obj))
      ProcessEvent(CurrentScreen->OnPress);

  // Object Event
  if (!TouchS.ActObjInfo.Obj)
    return;

  OnEvent(&TouchS.ActObjInfo, VTFT_EVT_PRESS);
}

static void Process_TP_Up() {
  char     isClick;
  TObjInfo actObj;

  // Screen Event
  if (CurrentScreen->Active)
    if ((CurrentScreen->SniffObjectEvents) || (!TouchS.ActObjInfo.Obj))
      ProcessEvent(CurrentScreen->OnUp);

  actObj = TouchS.ActObjInfo;
  // Cler active object info
  memset(&TouchS.ActObjInfo, 0, sizeof(TObjInfo));

  // Object Event
  if (!actObj.Obj)
    return;

  isClick = IsInsideObject(&actObj, TouchS.X, TouchS.Y);

  if (actObj.Flags & VTFT_INT_REPAINT_ON_UP)
    DrawScreen(CurrentScreen);

  OnEvent(&actObj, VTFT_EVT_UP);
  if (isClick)
    OnEvent(&actObj, VTFT_EVT_CLICK);
}

static void Process_TP_Down() {
  // Search for active object
  if (TouchS.Tag) {        // objects must not have zero for tag value
    if (TouchS.Tag != 255) // can not search objects by default tag value
      GetActiveObjectByTag(TouchS.Tag, &TouchS.ActObjInfo);
    if (!TouchS.ActObjInfo.Obj) // object not found by tag, search by coordinates
      GetActiveObjectByXY(TouchS.X, TouchS.Y, &TouchS.ActObjInfo);
  }

  // Screen Event
  if (CurrentScreen->Active)
    if ((CurrentScreen->SniffObjectEvents) || (!TouchS.ActObjInfo.Obj))
      ProcessEvent(CurrentScreen->OnDown);

  // Object Event
  if (!TouchS.ActObjInfo.Obj)
    return;

  if (TouchS.ActObjInfo.Flags & VTFT_INT_REPAINT_ON_DOWN)
    DrawScreen(CurrentScreen);

  OnEvent(&TouchS.ActObjInfo, VTFT_EVT_DOWN);
}

static void Process_TP_TagChange() {
  // Screen Event
  if (CurrentScreen->Active)
    ProcessEvent(CurrentScreen->OnTagChange);
}

void ProcessVTFTStack() {
  char         Tag, oldTag;
  unsigned int X, Y;

  oldTag = TouchS.Tag;

  if (FT800_Touch_GetTagXY(&X, &Y) == 1) {
    FT800_Touch_GetTag(&Tag);

    TouchS.Tag = Tag;
    TouchS.X = X;
    TouchS.Y = Y;

    if (!TouchS.Pressed) {
      TouchS.Pressed = 1;
      Process_TP_Down();
    }

    Process_TP_Press();
  }
  else if (TouchS.Pressed) {
    Process_TP_Up();

    TouchS.Tag = 0;
    TouchS.X   = X;
    TouchS.Y   = Y;

    TouchS.Pressed = 0;
  }

  if (oldTag != TouchS.Tag)
    Process_TP_TagChange();
}

static void InitObjects() {
  // Screen2: Init block start
  Screen2.Color             = 0xC618;
  Screen2.Width             = 480;
  Screen2.Height            = 272;
  Screen2.ObjectsCount      = 21;
  Screen2.LabelsCount       = 14;
  Screen2.Labels            = Screen2_Labels;
  Screen2.CirclesCount      = 3;
  Screen2.Circles           = Screen2_Circles;
  Screen2.EveSlidersCount   = 2;
  Screen2.EveSliders        = Screen2_EveSliders;
  Screen2.EveTextsCount     = 2;
  Screen2.EveTexts          = Screen2_EveTexts;
  Screen2.DynResStart       = 0;
  Screen2.Active            = 1;
  Screen2.SniffObjectEvents = 0;
  Screen2.OnUp              = 0;
  Screen2.OnDown            = 0;
  Screen2.OnTagChange       = 0;
  Screen2.OnPress           = 0;

  EveSlider_Basisachse.OwnerScreen      = &Screen2;
  EveSlider_Basisachse.Order            = 0;
  EveSlider_Basisachse.Visible          = 1;
  EveSlider_Basisachse.Opacity          = 255;
  EveSlider_Basisachse.Tag              = 255;
  EveSlider_Basisachse.Left             = 40;
  EveSlider_Basisachse.Top              = 152;
  EveSlider_Basisachse.Width            = 137;
  EveSlider_Basisachse.Height           = 19;
  EveSlider_Basisachse.Background_Color = 0x03DA;
  EveSlider_Basisachse.Thumb_Color      = 0xFFFF;
  EveSlider_Basisachse.Color            = 0x0148;
  EveSlider_Basisachse.Press_Color      = 0x7E3F;
  EveSlider_Basisachse.Value            = 0;
  EveSlider_Basisachse.Range            = 100;
  EveSlider_Basisachse.Flat             = 0;
  EveSlider_Basisachse.Active           = 1;
  EveSlider_Basisachse.OnUp             = 0;
  EveSlider_Basisachse.OnDown           = 0;
  EveSlider_Basisachse.OnClick          = 0;
  EveSlider_Basisachse.OnPress          = &EveSlider_Basisachse_OnPress;

  EveSlider_Basisachse_OnPress.Action       = EveSlider_BasisachseOnPress;
  EveSlider_Basisachse_OnPress.Sound.SndAct = VTFT_SNDACT_NONE;
  EveSlider_Basisachse_OnPress.Sound.Effect = _FT800_SOUND_XYLOPHONE;
  EveSlider_Basisachse_OnPress.Sound.Pitch  = _FT800_SOUND_PITCH_A5;
  EveSlider_Basisachse_OnPress.Sound.Volume = 255;

  EveSlider_Achse2.OwnerScreen      = &Screen2;
  EveSlider_Achse2.Order            = 1;
  EveSlider_Achse2.Visible          = 1;
  EveSlider_Achse2.Opacity          = 255;
  EveSlider_Achse2.Tag              = 255;
  EveSlider_Achse2.Left             = 42;
  EveSlider_Achse2.Top              = 227;
  EveSlider_Achse2.Width            = 140;
  EveSlider_Achse2.Height           = 19;
  EveSlider_Achse2.Background_Color = 0x03DA;
  EveSlider_Achse2.Thumb_Color      = 0xFFFF;
  EveSlider_Achse2.Color            = 0x0148;
  EveSlider_Achse2.Press_Color      = 0x7E3F;
  EveSlider_Achse2.Value            = 0;
  EveSlider_Achse2.Range            = 100;
  EveSlider_Achse2.Flat             = 0;
  EveSlider_Achse2.Active           = 1;
  EveSlider_Achse2.OnUp             = 0;
  EveSlider_Achse2.OnDown           = 0;
  EveSlider_Achse2.OnClick          = 0;
  EveSlider_Achse2.OnPress          = &EveSlider_Achse2_OnPress;

  EveSlider_Achse2_OnPress.Action       = EveSlider_Achse2OnPress;
  EveSlider_Achse2_OnPress.Sound.SndAct = VTFT_SNDACT_NONE;
  EveSlider_Achse2_OnPress.Sound.Effect = _FT800_SOUND_XYLOPHONE;
  EveSlider_Achse2_OnPress.Sound.Pitch  = _FT800_SOUND_PITCH_A5;
  EveSlider_Achse2_OnPress.Sound.Volume = 255;

  Circle2.OwnerScreen   = &Screen2;
  Circle2.Order         = 2;
  Circle2.Visible       = 1;
  Circle2.Opacity       = 255;
  Circle2.Tag           = 255;
  Circle2.Left          = 250;
  Circle2.Top           = 50;
  Circle2.Radius        = 70;
  Circle2.Pen_Width     = 1;
  Circle2.Pen_Color     = 0x0000;
  Circle2.Color         = 0xFFE0;
  Circle2.Press_Color   = 0xE71C;
  Circle2.Active        = 1;
  Circle2.OnUp          = 0;
  Circle2.OnDown        = 0;
  Circle2.OnClick       = 0;
  Circle2.OnPress       = 0;

  Circle4.OwnerScreen   = &Screen2;
  Circle4.Order         = 3;
  Circle4.Visible       = 1;
  Circle4.Opacity       = 255;
  Circle4.Tag           = 255;
  Circle4.Left          = 304;
  Circle4.Top           = 104;
  Circle4.Radius        = 16;
  Circle4.Pen_Width     = 1;
  Circle4.Pen_Color     = 0x0000;
  Circle4.Color         = 0xFFE0;
  Circle4.Press_Color   = 0xE71C;
  Circle4.Active        = 1;
  Circle4.OnUp          = 0;
  Circle4.OnDown        = 0;
  Circle4.OnClick       = 0;
  Circle4.OnPress       = 0;

  Circle_Blase.OwnerScreen   = &Screen2;
  Circle_Blase.Order         = 4;
  Circle_Blase.Visible       = 1;
  Circle_Blase.Opacity       = 255;
  Circle_Blase.Tag           = 255;
  Circle_Blase.Left          = 299;
  Circle_Blase.Top           = 121;
  Circle_Blase.Radius        = 10;
  Circle_Blase.Pen_Width     = 1;
  Circle_Blase.Pen_Color     = 0x0000;
  Circle_Blase.Color         = 0x8410;
  Circle_Blase.Press_Color   = 0xE71C;
  Circle_Blase.Active        = 1;
  Circle_Blase.OnUp          = 0;
  Circle_Blase.OnDown        = 0;
  Circle_Blase.OnClick       = 0;
  Circle_Blase.OnPress       = 0;

  EveText2.OwnerScreen = &Screen2;
  EveText2.Order       = 5;
  EveText2.Visible     = 1;
  EveText2.Opacity     = 255;
  EveText2.Tag         = 255;
  EveText2.Left        = 66;
  EveText2.Top         = 127;
  EveText2.Width       = 69;
  EveText2.Height      = 17;
  EveText2.Caption     = EveText2_Caption;
  EveText2.TextAlign   = taNone;
  EveText2.FontName    = 27;
  EveText2.Font_Color  = 0x0148;
  EveText2.FontHandle  = 27;
  EveText2.Source      = -1UL;
  EveText2.Active      = 1;
  EveText2.OnUp        = 0;
  EveText2.OnDown      = 0;
  EveText2.OnClick     = 0;
  EveText2.OnPress     = 0;

  EveText3.OwnerScreen = &Screen2;
  EveText3.Order       = 6;
  EveText3.Visible     = 1;
  EveText3.Opacity     = 255;
  EveText3.Tag         = 255;
  EveText3.Left        = 78;
  EveText3.Top         = 197;
  EveText3.Width       = 48;
  EveText3.Height      = 17;
  EveText3.Caption     = EveText3_Caption;
  EveText3.TextAlign   = taNone;
  EveText3.FontName    = 27;
  EveText3.Font_Color  = 0x0148;
  EveText3.FontHandle  = 27;
  EveText3.Source      = -1UL;
  EveText3.Active      = 1;
  EveText3.OnUp        = 0;
  EveText3.OnDown      = 0;
  EveText3.OnClick     = 0;
  EveText3.OnPress     = 0;

  Label10.OwnerScreen = &Screen2;
  Label10.Order       = 7;
  Label10.Visible     = 1;
  Label10.Opacity     = 255;
  Label10.Tag         = 255;
  Label10.Left        = 226;
  Label10.Top         = 216;
  Label10.Width       = 222;
  Label10.Height      = 17;
  Label10.Caption     = Label10_Caption;
  Label10.FontName    = Tahoma_13x16_Regular;
  Label10.Font_Color  = 0x0000;
  Label10.FontHandle  = 1;
  Label10.Source      = -1UL;
  Label10.Active      = 1;
  Label10.OnUp        = 0;
  Label10.OnDown      = 0;
  Label10.OnClick     = 0;
  Label10.OnPress     = 0;

  Label3.OwnerScreen = &Screen2;
  Label3.Order       = 8;
  Label3.Visible     = 1;
  Label3.Opacity     = 255;
  Label3.Tag         = 255;
  Label3.Left        = 38;
  Label3.Top         = 84;
  Label3.Width       = 26;
  Label3.Height      = 17;
  Label3.Caption     = Label3_Caption;
  Label3.FontName    = Tahoma_13x16_Regular;
  Label3.Font_Color  = 0x0000;
  Label3.FontHandle  = 1;
  Label3.Source      = -1UL;
  Label3.Active      = 1;
  Label3.OnUp        = 0;
  Label3.OnDown      = 0;
  Label3.OnClick     = 0;
  Label3.OnPress     = 0;

  Label4.OwnerScreen = &Screen2;
  Label4.Order       = 9;
  Label4.Visible     = 1;
  Label4.Opacity     = 255;
  Label4.Tag         = 255;
  Label4.Left        = 153;
  Label4.Top         = 39;
  Label4.Width       = 26;
  Label4.Height      = 17;
  Label4.Caption     = Label4_Caption;
  Label4.FontName    = Tahoma_13x16_Regular;
  Label4.Font_Color  = 0x0000;
  Label4.FontHandle  = 1;
  Label4.Source      = -1UL;
  Label4.Active      = 1;
  Label4.OnUp        = 0;
  Label4.OnDown      = 0;
  Label4.OnClick     = 0;
  Label4.OnPress     = 0;

  Label1.OwnerScreen = &Screen2;
  Label1.Order       = 10;
  Label1.Visible     = 1;
  Label1.Opacity     = 255;
  Label1.Tag         = 255;
  Label1.Left        = 38;
  Label1.Top         = 39;
  Label1.Width       = 26;
  Label1.Height      = 17;
  Label1.Caption     = Label1_Caption;
  Label1.FontName    = Tahoma_13x16_Regular;
  Label1.Font_Color  = 0x0000;
  Label1.FontHandle  = 1;
  Label1.Source      = -1UL;
  Label1.Active      = 1;
  Label1.OnUp        = 0;
  Label1.OnDown      = 0;
  Label1.OnClick     = 0;
  Label1.OnPress     = 0;

  Label5.OwnerScreen = &Screen2;
  Label5.Order       = 11;
  Label5.Visible     = 1;
  Label5.Opacity     = 255;
  Label5.Tag         = 255;
  Label5.Left        = 153;
  Label5.Top         = 62;
  Label5.Width       = 26;
  Label5.Height      = 17;
  Label5.Caption     = Label5_Caption;
  Label5.FontName    = Tahoma_13x16_Regular;
  Label5.Font_Color  = 0x0000;
  Label5.FontHandle  = 1;
  Label5.Source      = -1UL;
  Label5.Active      = 1;
  Label5.OnUp        = 0;
  Label5.OnDown      = 0;
  Label5.OnClick     = 0;
  Label5.OnPress     = 0;

  Label6.OwnerScreen = &Screen2;
  Label6.Order       = 12;
  Label6.Visible     = 1;
  Label6.Opacity     = 255;
  Label6.Tag         = 255;
  Label6.Left        = 153;
  Label6.Top         = 84;
  Label6.Width       = 26;
  Label6.Height      = 17;
  Label6.Caption     = Label6_Caption;
  Label6.FontName    = Tahoma_13x16_Regular;
  Label6.Font_Color  = 0x0000;
  Label6.FontHandle  = 1;
  Label6.Source      = -1UL;
  Label6.Active      = 1;
  Label6.OnUp        = 0;
  Label6.OnDown      = 0;
  Label6.OnClick     = 0;
  Label6.OnPress     = 0;

  Label2.OwnerScreen = &Screen2;
  Label2.Order       = 13;
  Label2.Visible     = 1;
  Label2.Opacity     = 255;
  Label2.Tag         = 255;
  Label2.Left        = 38;
  Label2.Top         = 62;
  Label2.Width       = 26;
  Label2.Height      = 17;
  Label2.Caption     = Label2_Caption;
  Label2.FontName    = Tahoma_13x16_Regular;
  Label2.Font_Color  = 0x0000;
  Label2.FontHandle  = 1;
  Label2.Source      = -1UL;
  Label2.Active      = 1;
  Label2.OnUp        = 0;
  Label2.OnDown      = 0;
  Label2.OnClick     = 0;
  Label2.OnPress     = 0;

  Label7.OwnerScreen = &Screen2;
  Label7.Order       = 14;
  Label7.Visible     = 1;
  Label7.Opacity     = 255;
  Label7.Tag         = 255;
  Label7.Left        = 24;
  Label7.Top         = 84;
  Label7.Width       = 8;
  Label7.Height      = 17;
  Label7.Caption     = Label7_Caption;
  Label7.FontName    = Tahoma_13x16_Regular;
  Label7.Font_Color  = 0x0000;
  Label7.FontHandle  = 1;
  Label7.Source      = -1UL;
  Label7.Active      = 1;
  Label7.OnUp        = 0;
  Label7.OnDown      = 0;
  Label7.OnClick     = 0;
  Label7.OnPress     = 0;

  Label8.OwnerScreen = &Screen2;
  Label8.Order       = 15;
  Label8.Visible     = 1;
  Label8.Opacity     = 255;
  Label8.Tag         = 255;
  Label8.Left        = 24;
  Label8.Top         = 39;
  Label8.Width       = 8;
  Label8.Height      = 17;
  Label8.Caption     = Label8_Caption;
  Label8.FontName    = Tahoma_13x16_Regular;
  Label8.Font_Color  = 0x0000;
  Label8.FontHandle  = 1;
  Label8.Source      = -1UL;
  Label8.Active      = 1;
  Label8.OnUp        = 0;
  Label8.OnDown      = 0;
  Label8.OnClick     = 0;
  Label8.OnPress     = 0;

  Label9.OwnerScreen = &Screen2;
  Label9.Order       = 16;
  Label9.Visible     = 1;
  Label9.Opacity     = 255;
  Label9.Tag         = 255;
  Label9.Left        = 24;
  Label9.Top         = 62;
  Label9.Width       = 8;
  Label9.Height      = 17;
  Label9.Caption     = Label9_Caption;
  Label9.FontName    = Tahoma_13x16_Regular;
  Label9.Font_Color  = 0x0000;
  Label9.FontHandle  = 1;
  Label9.Source      = -1UL;
  Label9.Active      = 1;
  Label9.OnUp        = 0;
  Label9.OnDown      = 0;
  Label9.OnClick     = 0;
  Label9.OnPress     = 0;

  Label11.OwnerScreen = &Screen2;
  Label11.Order       = 17;
  Label11.Visible     = 1;
  Label11.Opacity     = 255;
  Label11.Tag         = 255;
  Label11.Left        = 138;
  Label11.Top         = 84;
  Label11.Width       = 7;
  Label11.Height      = 17;
  Label11.Caption     = Label11_Caption;
  Label11.FontName    = Tahoma_13x16_Regular;
  Label11.Font_Color  = 0x0000;
  Label11.FontHandle  = 1;
  Label11.Source      = -1UL;
  Label11.Active      = 1;
  Label11.OnUp        = 0;
  Label11.OnDown      = 0;
  Label11.OnClick     = 0;
  Label11.OnPress     = 0;

  Label12.OwnerScreen = &Screen2;
  Label12.Order       = 18;
  Label12.Visible     = 1;
  Label12.Opacity     = 255;
  Label12.Tag         = 255;
  Label12.Left        = 138;
  Label12.Top         = 39;
  Label12.Width       = 9;
  Label12.Height      = 17;
  Label12.Caption     = Label12_Caption;
  Label12.FontName    = Tahoma_13x16_Regular;
  Label12.Font_Color  = 0x0000;
  Label12.FontHandle  = 1;
  Label12.Source      = -1UL;
  Label12.Active      = 1;
  Label12.OnUp        = 0;
  Label12.OnDown      = 0;
  Label12.OnClick     = 0;
  Label12.OnPress     = 0;

  Label13.OwnerScreen = &Screen2;
  Label13.Order       = 19;
  Label13.Visible     = 1;
  Label13.Opacity     = 255;
  Label13.Tag         = 255;
  Label13.Left        = 138;
  Label13.Top         = 62;
  Label13.Width       = 8;
  Label13.Height      = 17;
  Label13.Caption     = Label13_Caption;
  Label13.FontName    = Tahoma_13x16_Regular;
  Label13.Font_Color  = 0x0000;
  Label13.FontHandle  = 1;
  Label13.Source      = -1UL;
  Label13.Active      = 1;
  Label13.OnUp        = 0;
  Label13.OnDown      = 0;
  Label13.OnClick     = 0;
  Label13.OnPress     = 0;

  Label14.OwnerScreen = &Screen2;
  Label14.Order       = 20;
  Label14.Visible     = 1;
  Label14.Opacity     = 255;
  Label14.Tag         = 255;
  Label14.Left        = 24;
  Label14.Top         = 19;
  Label14.Width       = 148;
  Label14.Height      = 17;
  Label14.Caption     = Label14_Caption;
  Label14.FontName    = Tahoma_13x16_Regular;
  Label14.Font_Color  = 0x0000;
  Label14.FontHandle  = 1;
  Label14.Source      = -1UL;
  Label14.Active      = 1;
  Label14.OnUp        = 0;
  Label14.OnDown      = 0;
  Label14.OnClick     = 0;
  Label14.OnPress     = 0;

}

void Init_MCU() {
  // Setup FT800 SPI interface
  SPI3_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 64, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);
}

void InitVTFTStack() {
  Init_MCU();

  SPI_Set_Active(SPI3_Read, SPI3_Write);

  FT800_Init();

  // Core setup
  FT800_Core_ClockSource(_FT800_CLK_SOURCE_EXTERNAL);
  FT800_Core_ClockPLL(_FT800_CLK_PLL_48MHz);

  // Internal modules setup
  FT800_Display_SetConfig(&VTFT_FT800_CONFIG_DISPLAY);

  FT800_PWM_SetConfig(&VTFT_FT800_CONFIG_PWM);

  FT800_GPIO_SetConfig(&VTFT_FT800_CONFIG_GPIO);

  FT800_Audio_SetConfig(&VTFT_FT800_CONFIG_AUDIO);

  FT800_Sound_SetConfig(&VTFT_FT800_CONFIG_SOUND);

  FT800_Interrupt_SetConfig(&VTFT_FT800_CONFIG_INTERRUPT);

  FT800_Touch_SetConfig(&VTFT_FT800_CONFIG_TOUCH);

  FT800_TouchTransform_SetConfig(&VTFT_FT800_CONFIG_TOUCHTRANSFORM);

  // Init all dynamic objects
  InitObjects();

  // Draw start screen
  DrawScreen(&Screen2);
}
